package uoc.com.mimenudehoy.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import uoc.com.mimenudehoy.mappings.EnumCuisineTypePropertyEditor;
import uoc.com.mimenudehoy.model.EnumCuisineType;
import uoc.com.mimenudehoy.model.Recipe;
import uoc.com.mimenudehoy.model.form.SearchRecipeForm;
import uoc.com.mimenudehoy.service.RecipeService;

@Controller
public class SearchRecipeController {

	@Autowired
	private RecipeService service;

	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(EnumCuisineType.class, new EnumCuisineTypePropertyEditor());
	}		
	
	
	@RequestMapping(value = { "/recipes" })
	public String list(Map<String, Object> map) {

		List<Recipe> recipes = service.getLastRecipes();
		map.put("list", recipes);
		
		map.put("allCuisineTypes", EnumCuisineType.values() );
		
		map.put("recipe", new SearchRecipeForm() );
		map.put("menu", "recipes");
		
		List<Recipe> lateral = service.getHighLightsLateral();
		map.put("recipesLateral", lateral);

		
		return "recipes";
	}
	
	
	@RequestMapping("/recipe/{id}")
	public String getRecipe(@PathVariable String id, Map<String, Object> map) {

		Recipe obj = service.getRecipe(id);

		map.put("recipe", obj);
		map.put("menu", "recipes");

		List<Recipe> lateral = service.getHighLightsLateral();
		map.put("recipesLateral", lateral);
		
		return "viewRecipe";
	}
	
	@RequestMapping(method = RequestMethod.POST, value = { "/searchRecipe" })
	public String search(@ModelAttribute("recipe") SearchRecipeForm form , Map<String, Object> map) {

		String text = form.getText();
		EnumCuisineType tc = form.getCuisine();
		
		List<Recipe> recipes = service.searchRecipes(text, tc);
		
		map.put("list", recipes);
		map.put("allCuisineTypes", EnumCuisineType.values() );
		map.put("recipe", form );
		
		map.put("menu", "recipes");
		List<Recipe> lateral = service.getHighLightsLateral();
		map.put("recipesLateral", lateral);

		return "recipes";
	}
	
}
