package uoc.com.mimenudehoy.controller.admin;

import java.util.List;
import java.util.Map;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import uoc.com.mimenudehoy.mappings.EnumCourseTypePropertyEditor;
import uoc.com.mimenudehoy.mappings.EnumDishTypePropertyEditor;
import uoc.com.mimenudehoy.mappings.IngredientPropertyEditor;
import uoc.com.mimenudehoy.model.Dish;
import uoc.com.mimenudehoy.model.EnumCourseType;
import uoc.com.mimenudehoy.model.EnumDishType;
import uoc.com.mimenudehoy.model.Ingredient;
import uoc.com.mimenudehoy.service.DishService;
import uoc.com.mimenudehoy.service.IngredientService;

@Controller
public class DishesController {

	@Autowired
	private DishService service;
	
	@Autowired
	private IngredientService ingService;
	

	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(EnumDishType.class, new EnumDishTypePropertyEditor());
	    binder.registerCustomEditor(EnumCourseType.class, new EnumCourseTypePropertyEditor());
	    binder.registerCustomEditor(Ingredient.class, new IngredientPropertyEditor(ingService.listIngredients()));
	}	

	@RequestMapping(value = { "/admin/dish/list" })
	public String list(Map<String, Object> map) {

		map.put("dish", new Dish());
		
		
		List<Dish> dishes = service.listDishes();
		for(Dish dish:dishes){
			dish.getIngredients();
		}
		
		map.put("list", dishes);

		
		
/*		
		map.put("listDishTypes", EnumDishType.values());
		map.put("listCourseTypes", EnumCourseType.values());
		map.put("allIngredients", ingService.listIngredients());
*/
		return "listDishes";
	}

	@RequestMapping("/admin/dish/{id}")
	public String showDish(@PathVariable String id, Map<String, Object> map) {

		Dish obj = id.equals("new")?new Dish():service.getDish(id);

		map.put("dish", obj);
		
		// initialize lazy
		Hibernate.initialize(obj.getIngredients());
		/*
		for(Ingredient ing:ings){
			System.out.println(ing);
		}
		*/
		
		map.put("listDishTypes", EnumDishType.values());
		map.put("listCourseTypes", EnumCourseType.values());
		map.put("allIngredients", ingService.listIngredients());
		
		return "dishFormDlg";
	}

	@RequestMapping(value = "/admin/dish/save", method = RequestMethod.POST)
	public String save(@ModelAttribute("dish") Dish vo, BindingResult result) {

		service.save(vo);

		return "redirect:./list";
	}

	@RequestMapping("/admin/dish/delete/{id}")
	public String delete(@PathVariable("id") String id) {

		service.delete(id);

		return "redirect:../list";
	}
}
