package uoc.com.mimenudehoy.controller.admin;

import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import uoc.com.mimenudehoy.mappings.EnumIntolleranceTypePropertyEditor;
import uoc.com.mimenudehoy.mappings.EnumOriginTypePropertyEditor;
import uoc.com.mimenudehoy.model.EnumIntoleranceType;
import uoc.com.mimenudehoy.model.EnumOriginType;
import uoc.com.mimenudehoy.model.Ingredient;
import uoc.com.mimenudehoy.model.validator.IngredientValidator;
import uoc.com.mimenudehoy.service.IngredientService;

@Controller
public class IngredientsController {

	@Autowired
	private IngredientService service;

	@Autowired
    private IngredientValidator ingValidator;	
	
	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(EnumIntoleranceType.class, new EnumIntolleranceTypePropertyEditor());
	    binder.registerCustomEditor(EnumOriginType.class, new EnumOriginTypePropertyEditor());
	    binder.setValidator(ingValidator);
	}
		

	@RequestMapping(value = { "/admin/ingredient/list" })
	public String listIngredients(Map<String, Object> map) {

		map.put("elem", new Ingredient());
		map.put("list", service.listIngredients());

		return "listIngredients";
	}

	@RequestMapping("/admin/ingredient/{id}")
	public String getIngredient(@PathVariable String id, Map<String, Object> map) {

		Ingredient obj = id.equals("new")?new Ingredient():service.getIngredient(id);

		map.put("elem", obj);

		map.put("listIntoleranceTypes", EnumIntoleranceType.values());		
		map.put("listOriginTypes", EnumOriginType.values());
				
		return "ingredientFormDlg";
	}

	@RequestMapping(value = "/admin/ingredient/save", method = RequestMethod.POST)
	public String saveIngredient(@ModelAttribute("elem") @Validated Ingredient obj, BindingResult result , Map<String, Object> map) {

		if (result.hasErrors()) {
			map.put("listIntoleranceTypes", EnumIntoleranceType.values());		
			map.put("listOriginTypes", EnumOriginType.values());

            return "ingredientFormDlg";
        }
		service.save(obj);

		return "redirect:list";
	}

	@RequestMapping("/admin/ingredient/delete/{id}")
	public String deleteIngredient(@PathVariable("id") String id) {

		service.delete(id);

		return "redirect:../list";
	}
}
