package uoc.com.mimenudehoy.controller.admin;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import uoc.com.mimenudehoy.mappings.DishPropertyEditor;
import uoc.com.mimenudehoy.mappings.EnumMenuTypePropertyEditor;
import uoc.com.mimenudehoy.model.Dish;
import uoc.com.mimenudehoy.model.EnumCourseType;
import uoc.com.mimenudehoy.model.EnumMenuType;
import uoc.com.mimenudehoy.model.Menu;
import uoc.com.mimenudehoy.model.MenuDaily;
import uoc.com.mimenudehoy.model.Recipe;
import uoc.com.mimenudehoy.model.Subscription;
import uoc.com.mimenudehoy.model.User;
import uoc.com.mimenudehoy.service.DishService;
import uoc.com.mimenudehoy.service.MenuDailyService;
import uoc.com.mimenudehoy.service.MenuService;
import uoc.com.mimenudehoy.service.RecipeService;
import uoc.com.mimenudehoy.service.SubscriptionsService;
import uoc.com.mimenudehoy.service.UserService;

@Controller
@SessionAttributes("subscription")
public class MenuController {

	@Autowired
	private MenuService service;

	@Autowired
	private MenuDailyService mdService;
		
	@Autowired
	private SubscriptionsService subsService;

	@Autowired
	private DishService dishService;

	@Autowired
	private UserService userService;

	@Autowired
	private RecipeService recipeService;

	
	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(Dish.class, new DishPropertyEditor(dishService.listDishes()));
	    binder.registerCustomEditor(EnumMenuType.class, new EnumMenuTypePropertyEditor());
	}	
	
	
	@RequestMapping(value = { "/admin/menu/list" })
	public String listMenus(Map<String, Object> map) {

		map.put("obj", new Menu());
		map.put("list", service.listMenus());

/*
		map.put("listStarter", dishService.getDishesByType(EnumCourseType.STARTER));
		map.put("listMain", dishService.getDishesByType(EnumCourseType.MAIN));
		map.put("listDessert", dishService.getDishesByType(EnumCourseType.DESSERT));
		map.put("listMenuTypes", EnumMenuType.values());
*/
		
		return "listMenus";
	}

	@RequestMapping("/admin/menu/{id}")
	public String show(@PathVariable String id, Map<String, Object> map) {

		Menu obj = (id.equals("new"))?new Menu():service.getMenu(id);

		map.put("obj", obj);

		map.put("listStarter", dishService.getDishesByType(EnumCourseType.STARTER));
		map.put("listMain", dishService.getDishesByType(EnumCourseType.MAIN));
		map.put("listDessert", dishService.getDishesByType(EnumCourseType.DESSERT));
		map.put("listMenuTypes", EnumMenuType.values());
		
		
		return "menuFormDlg";
	}

	@RequestMapping(value = "/admin/menu/save", method = RequestMethod.POST)
	public String save(@ModelAttribute("obj") Menu obj, BindingResult result) {

		service.save(obj);

		return "redirect:list";
	}

	@RequestMapping("/admin/menu/delete/{id}")
	public String delete(@PathVariable("id") String id) {

		service.delete(id);

		return "redirect:../list";
	}
	

	@RequestMapping(value = { "/admin/menu/asign" })
	public String assignMenu(Map<String, Object> map) {

		map.put("list", subsService.listMissingMenus());

		return "assignMenus";
	}


	@RequestMapping(value = { "/admin/menu/asign/{id_subscription}" })
	public String assignMenu(@PathVariable String id_subscription , Map<String, Object> map, ModelMap model) {

		
		Subscription subs = subsService.getSubscription(id_subscription);
		map.put("subscription", subs);
		
		// guardamos en sesion
		model.addAttribute("subscription", subs);
		
		
		map.put("list", subsService.getAllDailyMenus(id_subscription));

		MenuDaily md = new MenuDaily();
		md.setSubscription(subs);
		map.put("md", new MenuDaily());
		
		return "subscriptionMenus";
	}

	@RequestMapping(value = { "/admin/menu/select/{type}" })
	public String selectMenus(@PathVariable EnumMenuType type, @RequestParam("fecha") String fecha, @RequestParam("idmd") String idmd, Map<String, Object> map) {
		//EnumMenuType tipo = EnumMenuType.valueOf(type);
		map.put("list", service.listMenusByType(type));
	
		map.put("fecha", fecha);
		map.put("idmd", idmd);
		
		return "selectMenus";
	}


	@RequestMapping(value = "/admin/menudaily/save", method = RequestMethod.POST)
	public String saveMenuDaily(@RequestParam("menu") String id_menu, @RequestParam("fecha") String fechaYYYYMMDD, @RequestParam("idmd") String idmd, ModelMap model) throws ParseException {

		MenuDaily md = new MenuDaily();

		Subscription subs = (Subscription) model.get("subscription");
				
		if(!idmd.isEmpty()){
			md = mdService.find(idmd);
		 }
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
		Date fecha =sdf.parse(fechaYYYYMMDD);
		md.setDate(fecha);

		Menu menu = service.getMenu(id_menu);
		md.setMenu(menu);
		
		
		md.setSubscription(subs);
		service.saveMenuDaily(md);

		model.put("obj", md);
		
		return "dailymenuForm";
		
	}

	@RequestMapping(value = { "/myMenus" })
	public String myMenus(Map<String, Object> map) {

		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		String username = auth.getName(); 
		
		User user = userService.getUser(username);
		Subscription subs = subsService.listActivesByUser(user).get(0);
		
		map.put("sub", subs);
		
		List<MenuDaily> mds = mdService.getDailyMenusOfThisWeek(subs);
		//System.out.println("total:" + mds.size());
		map.put("menus_thisWeek", mds );
		
		mds = mdService.getDailyMenusOfNextWeek(subs);
		//System.out.println("total:" + mds.size());
		map.put("menus_nextWeek", mds );
		
		// web design requirements
		map.put("menu", "myMenus");
		//List<Recipe> lateral = recipeService.getHighLightsLateral();
		//map.put("recipesLateral", lateral);

		
		return "myMenus";
	}
	
}
