package uoc.com.mimenudehoy.controller.admin;

import java.sql.Blob;
import java.sql.SQLException;
import java.util.Date;
import java.util.Map;

import javax.sql.rowset.serial.SerialException;

import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import org.springframework.web.multipart.support.ByteArrayMultipartFileEditor;

import uoc.com.mimenudehoy.mappings.DishPropertyEditor;
import uoc.com.mimenudehoy.mappings.EnumCuisineTypePropertyEditor;
import uoc.com.mimenudehoy.model.Dish;
import uoc.com.mimenudehoy.model.EnumCuisineType;
import uoc.com.mimenudehoy.model.Recipe;
import uoc.com.mimenudehoy.service.DishService;
import uoc.com.mimenudehoy.service.RecipeService;

@Controller
public class RecipeController {

	@Autowired
	private RecipeService service;
	@Autowired
	private DishService dishService;

	
	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(EnumCuisineType.class, new EnumCuisineTypePropertyEditor());
	    binder.registerCustomEditor(Dish.class, new DishPropertyEditor(dishService.listDishes()));
	    
	    // Convert multipart object to byte[]
		binder.registerCustomEditor(byte[].class, new ByteArrayMultipartFileEditor());
		
	}		
	
	@RequestMapping("/admin/recipe/list")
	public String list(Map<String, Object> map) {

		map.put("recipe", new Recipe());
		map.put("recipes", service.listRecipes());
		

		return "listRecipes";
	}

	@RequestMapping("/admin/recipe/{id}")
	public String getRecipe(@PathVariable String id, Map<String, Object> map) {

		
		Recipe user = (id.equals("new"))?new Recipe():service.getRecipe(id);
		map.put("recipe", user);

		map.put("allCuisineTypes", EnumCuisineType.values());
		map.put("listDishes", dishService.listDishes());
		
		return "recipeFormDlg";
	}

	@RequestMapping(value="/admin/recipe/save", method = RequestMethod.POST)
	public String save(@ModelAttribute("recipe") Recipe rec, @RequestParam CommonsMultipartFile[] fileUpload , BindingResult result) throws SerialException, SQLException {

		if(rec.getDate()==null) rec.setDate(new Date());
		
		if (fileUpload != null && fileUpload.length > 0) {
			int pos=0;
            for (CommonsMultipartFile aFile : fileUpload){
                  
                //System.out.println("Saving file: " + aFile.getOriginalFilename());
            	Blob blob = new javax.sql.rowset.serial.SerialBlob(aFile.getBytes());
                if(pos==0){
                    rec.setPhotoFileName(aFile.getOriginalFilename());
                    rec.setPhoto(blob);                	
                } else if ( pos==1){
                    rec.setThumbFileName(aFile.getOriginalFilename());
                    rec.setThumb(blob);
                }
                pos++;
            }
        }
		
		
		service.save(rec);

		return "redirect:./list";
	}

	@RequestMapping("/admin/recipe/delete/{id}")
	public String delete(@PathVariable("id") String id) {

		service.delete(id);

		/*
		 * redirects to the path relative to the current path
		 */
		// return "redirect:../listBooks";

		/*
		 * Note that there is the slash "/" right after "redirect:" So, it
		 * redirects to the path relative to the project root path
		 */
		return "redirect:../list";
	}
}
