package uoc.com.mimenudehoy.controller.admin;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.SessionAttributes;

import uoc.com.mimenudehoy.mappings.DateDDMMYYYYPropertyEditor;
import uoc.com.mimenudehoy.mappings.EnumMenuTypePropertyEditor;
import uoc.com.mimenudehoy.mappings.UserPropertyEditor;
import uoc.com.mimenudehoy.model.EnumMenuType;
import uoc.com.mimenudehoy.model.EnumRoleAuthority;
import uoc.com.mimenudehoy.model.Recipe;
import uoc.com.mimenudehoy.model.Role;
import uoc.com.mimenudehoy.model.Subscription;
import uoc.com.mimenudehoy.model.User;
import uoc.com.mimenudehoy.service.RecipeService;
import uoc.com.mimenudehoy.service.SubscriptionsService;
import uoc.com.mimenudehoy.service.UserService;

@Controller
@SessionAttributes("subscription")
public class SubscriptionsController {

	@Autowired
	private SubscriptionsService service;

	@Autowired
	private UserService usersService;

	@Autowired
	private RecipeService recipeService;

	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(User.class, new UserPropertyEditor(usersService.listUsers()));
	    binder.registerCustomEditor(EnumMenuType.class, new EnumMenuTypePropertyEditor());
	    binder.registerCustomEditor(Date.class, new DateDDMMYYYYPropertyEditor());
	}
	
	@RequestMapping(value = { "/admin/subscription/list" })
	public String listSubscriptions(Map<String, Object> map) {

		map.put("elem", new Subscription());
		map.put("list", service.list());
		

		return "listSubscriptions";
	}

	@RequestMapping("/admin/subscription/{id}")
	public String getSubscription(@PathVariable String id, Map<String, Object> map) {

		Subscription elem = new Subscription();
		if(!id.equals("new")) {
			elem = service.getSubscription(id);
		}

		map.put("elem", elem);

		map.put("listMenuTypes", EnumMenuType.values());		
		map.put("allUsers", usersService.listUsers() );
		
		
		return "subscriptionForm";
	}

	@RequestMapping(value = "/admin/subscription/save", method = RequestMethod.POST)
	public String save(@ModelAttribute("elem") Subscription elem, BindingResult result) {

		service.save(elem);

		return "redirect:list";
	}

	@RequestMapping("/admin/subscription/delete/{id}")
	public String delete(@PathVariable("id") String id) {

		service.delete(id);

		return "redirect:../list";
	}
	
	
	@RequestMapping(value = { "/suscribe" })
	public String suscribe(Map<String, Object> map) {

		List<Map> tmp = new ArrayList<Map>(); 
		
		for(EnumMenuType tipo : EnumMenuType.values()){
			Map elem = new HashMap();
			elem.put("tipo",tipo);
			elem.put("precio", tipo.getCost());
			tmp.add(elem);
		}
		map.put("list", tmp);
		map.put("subscription", new Subscription());
		map.put("listMenuTypes", EnumMenuType.values());

		// web requirements
		map.put("menu", "suscribe");
		List<Recipe> lateral = recipeService.getHighLightsLateral();
		map.put("recipesLateral", lateral);

		
		return "suscribe";
	}

	@RequestMapping(value = { "/register" })
	public String suscribe(@ModelAttribute("subscription") Subscription sus, Map<String, Object> map, ModelMap model) {
	
		map.put("user", new User());
		map.put("menu", "suscribe");		

		return "suscribe_2";
	}

	@RequestMapping(value = { "/registered" })
	public String register(@ModelAttribute("user") User user, @ModelAttribute("subscription") Subscription subs, Map<String, Object> map, ModelMap model) {

		EnumMenuType tipo = subs.getMenuType();

		user.setEnabled(true);
		usersService.saveClient(user);
		
		/*
		// grabar el usuario como cliente
		Role roleClient = usersService.saveClient(user);getRole(EnumRoleAuthority.CLIENT);
		Hibernate.initialize(roleClient);
		System.out.println(roleClient);
		
		user.setEnabled(true);
		Hibernate.initialize(user);
		user.getRoles().add(roleClient);
		
		//usersService.saveClient(user);

		usersService.saveUser(user);
		*/
		
		
		//Subscription subs = new Subscription();
		subs.setMenuType(tipo);
		subs.setUser(user);

		// fecha Fin de suscripcion es la actual + 1 mes
		Calendar cal = Calendar.getInstance();
		cal.setTime(subs.getStart_date());
		cal.add(Calendar.MONTH,1);
		
		//reset time part
		cal.set(Calendar.HOUR,0);
		cal.set(Calendar.MINUTE,0);
		cal.set(Calendar.SECOND,0);
		cal.set(Calendar.MILLISECOND,0);
		
		subs.setEnd_date(cal.getTime());
		subs.setCost(tipo.getCost());
		
		service.save(subs);
		
		map.put("subs", subs);
		map.put("user", user);
		
		Authentication auth = new UsernamePasswordAuthenticationToken(user.getUsername(),  user.getPassword(), user.getAuthorities());

		SecurityContextHolder.getContext().setAuthentication(auth);
		
		return "suscribe_3";
	}
	
}
