package uoc.com.mimenudehoy.controller.admin;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import uoc.com.mimenudehoy.mappings.RolePropertyEditor;
import uoc.com.mimenudehoy.model.EnumRoleAuthority;
import uoc.com.mimenudehoy.model.Recipe;
import uoc.com.mimenudehoy.model.Role;
import uoc.com.mimenudehoy.model.Subscription;
import uoc.com.mimenudehoy.model.User;
import uoc.com.mimenudehoy.model.validator.UserValidator;
import uoc.com.mimenudehoy.service.RecipeService;
import uoc.com.mimenudehoy.service.UserService;

@Controller
public class UserController {

	@Autowired
	private UserValidator userValidator;
	
	@Autowired
	private UserService userService;

	@Autowired
	private RecipeService recipeService;
	
	
	@RequestMapping({"/admin"})
	public String adminPanelDefault(Map<String, Object> map) {
		
		
		// NUTRICIONISTA debe ir a los menus
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		for(GrantedAuthority ga:auth.getAuthorities()){
			if(ga.getAuthority().equals(EnumRoleAuthority.ADMIN.getValue())){
				return "redirect:/admin/user/list";
			}
		}
		
		return "redirect:/admin/menu/list";
	}

	
	@RequestMapping({"/admin/user/list"})
	public String listUsers(Map<String, Object> map) {

		map.put("user", new User());
		
		List<User> listado = userService.listUsers();
		map.put("userList", listado);
		map.put("roleList", userService.listRoles());
		
		return "listUsers";
	}

	@InitBinder
	public void initBinder(WebDataBinder binder) {
	    binder.registerCustomEditor(Role.class, new RolePropertyEditor(userService.listRoles()));
	    binder.setValidator(userValidator);
	}
	
	@RequestMapping("/admin/user/{userId}")
	public String getUser(@PathVariable String userId, Map<String, Object> map) {

		User user = new User();
		if(!userId.equals("new")) {
			user = userService.getUser(userId);
		}

		List<Role> roles = userService.listRoles();
		
		Hibernate.initialize(user.getRoles());
		Set<Role> userRoles = user.getRoles();
		//System.out.println("user roles:" + userRoles.size());
		
		map.put("user", user);
		map.put("roleList", roles);
		
		return "userFormDlg";
	}

	@RequestMapping(value="/admin/user/save", method = RequestMethod.POST)
	public String saveUser(@ModelAttribute("user") @Validated User user, BindingResult result, Map<String, Object> map) {

		if(result.hasErrors()){
			List<Subscription> subs = userService.getSubscriptions(user);
			map.put("user", user);

			List<Role> roles = userService.listRoles();
			map.put("roleList", roles);
			
			return "userFormDlg";	
		}

		// if username has changed delete previous username
		if(user.getUsernameLast()!=null && !user.getUsernameLast().isEmpty() && !user.getUsername().equals(user.getUsernameLast())){
			userService.deleteUser(user.getUsernameLast());
		}
		
		userService.saveUser(user);

		return "redirect:list";
	}


	
	
	@RequestMapping("/admin/user/delete/{name}")
	public String deleteUser(@PathVariable("name") String name) {

		userService.deleteUser(name);

		return "redirect:../list";
	}
	
	@RequestMapping("/myProfile")
	public String getUser(Map<String, Object> map) {

		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		String userName = auth.getName(); 
		User user = userService.getUser(userName);

		List<Subscription> subs = userService.getSubscriptions(user);

		map.put("user", user);
		map.put("subscriptions" , subs);
		
		map.put("menu" , "myProfile");
		List<Recipe> lateral = recipeService.getHighLightsLateral();
		map.put("recipesLateral", lateral);

		return "myProfile";
	}

	@RequestMapping(value="/myProfile/save", method = RequestMethod.POST)
	public String saveProfile(@ModelAttribute("user") @Validated User user, BindingResult result, Map<String, Object> map) {

		if(result.hasErrors()){
			List<Subscription> subs = userService.getSubscriptions(user);
			map.put("user", user);
			map.put("subscriptions" , subs);
			
			map.put("menu" , "myProfile");
			List<Recipe> lateral = recipeService.getHighLightsLateral();
			map.put("recipesLateral", lateral);

			return "myProfile";	
		}
		
		userService.saveUser(user);

		return "redirect:myMenus";
	}

	
	
}
