package uoc.com.mimenudehoy.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;

import org.springframework.format.annotation.NumberFormat;
import org.springframework.format.annotation.NumberFormat.Style;



@Entity
@Table(name = "dishes")
public class Dish implements Serializable {

	private static final long serialVersionUID = -7990757853291768324L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id;
	
	@Column(length = 255)
	private String name;
	
	@Column(length = 512)
	private String description;
	
	@NumberFormat(style=Style.NUMBER, pattern="###.##")
	private Double price;

	@Enumerated(EnumType.ORDINAL)
	private EnumDishType category;
	
	@Enumerated(EnumType.ORDINAL)
	private EnumCourseType course;
	
	private Integer calories;
	
	 @ManyToMany(cascade=CascadeType.REFRESH)  
	 @JoinTable(name="dishes_ingredients", joinColumns=@JoinColumn(name="dish_id"), inverseJoinColumns=@JoinColumn(name="ingredient_id"))  
    private List<Ingredient> ingredients = new ArrayList<Ingredient>();

	
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	
	public Integer getCalories() {
		return calories;
	}

	public void setCalories(Integer calories) {
		this.calories = calories;
	}

	@Column(nullable = false)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Double getPrice() {
		return price;
	}

	public void setPrice(Double price) {
		this.price = price;
	}

	public EnumCourseType getCourse() {
		return course;
	}

	public void setCourse(EnumCourseType course) {
		this.course = course;
	}

	public EnumDishType getCategory() {
		return category;
	}

	public void setCategory(EnumDishType category) {
		this.category = category;
	}

	public List<Ingredient> getIngredients() {
		return ingredients;
	}

	public void setIngredients(List<Ingredient> ingredients) {
		this.ingredients = ingredients;
	}

	@Override
    public boolean equals(Object obj){
    	if (!(obj instanceof Dish)) return false;
    	Dish u = (Dish) obj;
    	return this.getId().equals(u.getId());
    }

}
