package uoc.com.mimenudehoy.model;

import java.io.Serializable;
import java.sql.Blob;
import java.sql.SQLException;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.codec.binary.Base64;

@Entity
@Table(name = "recipes")
public class Recipe implements Serializable {


	private static final long serialVersionUID = -469250495752055302L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id;
	
	@Column(length=255)
	private String name;

	@Column(length=255)
	private String author;
	
	private Date date;
	
	@Column(columnDefinition="TEXT")
	private String preparation;

	@Enumerated(EnumType.ORDINAL)
	private EnumCuisineType cuisine;
	
	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name="dish")	
	private Dish dish;
	
	@Column(nullable=true)
	private Blob  photo;
	
	@Column(nullable=true)
	private String photoFileName;
	

	@Column(nullable=true)
	private Blob thumb;
	
	@Column(nullable=true)
	private String thumbFileName;
	

	private boolean highlightCentral;
	private boolean highlightFooter;
	private boolean highlightLateral;

	
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getPreparation() {
		return preparation;
	}

	public void setPreparation(String preparation) {
		this.preparation = preparation;
	}

	public EnumCuisineType getCuisine() {
		return cuisine;
	}

	public void setCuisine(EnumCuisineType cuisine) {
		this.cuisine = cuisine;
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
	}


	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "dish", nullable = false)  
	public Dish getDish() {
		return dish;
	}

	public void setDish(Dish dish) {
		this.dish = dish;
	}

	@Override
    public boolean equals(Object obj){
    	if (!(obj instanceof Recipe)) return false;
    	Recipe u = (Recipe) obj;
    	return this.getId().equals(u.getId());
    }

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public Blob getPhoto() {
		return photo;
	}

	public void setPhoto(Blob photo) {
		this.photo = photo;
	}

	public String getPhotoFileName() {
		return photoFileName;
	}

	public void setPhotoFileName(String photoFileName) {
		this.photoFileName = photoFileName;
	}

	public Blob getThumb() {
		return thumb;
	}

	public void setThumb(Blob thumb) {
		this.thumb = thumb;
	}

	public String getThumbFileName() {
		return thumbFileName;
	}

	public void setThumbFileName(String thumbFileName) {
		this.thumbFileName = thumbFileName;
	}

	public boolean isHighlightCentral() {
		return highlightCentral;
	}

	public void setHighlightCentral(boolean highlightCentral) {
		this.highlightCentral = highlightCentral;
	}

	public boolean isHighlightFooter() {
		return highlightFooter;
	}

	public void setHighlightFooter(boolean highlightFooter) {
		this.highlightFooter = highlightFooter;
	}

	public boolean isHighlightLateral() {
		return highlightLateral;
	}

	public void setHighlightLateral(boolean highlightLateral) {
		this.highlightLateral = highlightLateral;
	}


	@Transient
	public String getPhotoBase64() throws SQLException{
		if(this.getPhoto()!=null){
			Blob blob = this.getPhoto();
			int blobLength = (int) blob.length();  
			byte[] blobAsBytes = blob.getBytes(1, blobLength);
			byte[] encoded=Base64.encodeBase64(blobAsBytes);
			String encodedString = new String(encoded);
			return encodedString;
		}
		return "";
	}

	@Transient
	public String getThumbBase64() throws SQLException{
		if(this.getThumb()!=null){
			Blob blob = this.getThumb();
			int blobLength = (int) blob.length();  
			byte[] blobAsBytes = blob.getBytes(1, blobLength);
			byte[] encoded=Base64.encodeBase64(blobAsBytes);
			String encodedString = new String(encoded);
			return encodedString;
		}
		return "";
	}


}
