package uoc.com.mimenudehoy.model;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

@Entity
@Table(name="users")
public class User implements Serializable, UserDetails{
     
	private static final long serialVersionUID = -4216276596623712295L;

	@Id
	@Column(length=100, unique=true)
    private String username;
     
	@Transient
	private String usernameLast;
	
	@Column(nullable=false)
    private String password;
    
    private Boolean enabled = true;

    private String address;
    
    private String email;
    
    private Integer age;
    
    private String preferences;
    
    private String name;
    private String lastName;
    
    
    @ManyToMany(cascade=CascadeType.REFRESH, fetch=FetchType.EAGER)  
    @JoinTable(name="users_roles",  
    joinColumns={@JoinColumn(name="user_username", referencedColumnName="username",unique=false)},  
    inverseJoinColumns={@JoinColumn(name="roles_id", referencedColumnName="id",unique=false)})  
    private Set<Role> roles = new HashSet<Role>();
    
    @OneToMany(fetch=FetchType.LAZY , cascade=CascadeType.ALL)
    @JoinColumn(name="user")
	private Set<Subscription> subscriptions;
    
    
	public Set<Role> getRoles() {
		return this.roles;
	};

    public String getPassword() {
        return password;
    }
 
    public void setPassword(String password) {
        this.password = password;
    }
 

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public Boolean getEnabled() {
		return enabled;
	}

	public void setEnabled(Boolean enabled) {
		this.enabled = enabled;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public Integer getAge() {
		return age;
	}

	public void setAge(Integer age) {
		this.age = age;
	}

	public String getPreferences() {
		return preferences;
	}

	public void setPreferences(String preferences) {
		this.preferences = preferences;
	}

	public void setRoles(Set<Role> roles) {
		this.roles = roles;
	}
 
	@Override
    public boolean equals(Object obj){
    	if (!(obj instanceof User)) return false;
    	User u = (User) obj;
    	return this.getUsername().equals(u.getUsername());
    }

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Collection<Role> getAuthorities() {
		return roles ;
	}
	public boolean isAccountNonExpired() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isAccountNonLocked() {
		// TODO Auto-generated method stub
		return (enabled=true);
	}

	public boolean isCredentialsNonExpired() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isEnabled() {
		return enabled;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public Set<Subscription> getSubscriptions() {
		return subscriptions;
	}

	public void setSubscriptions(Set<Subscription> subscriptions) {
		this.subscriptions = subscriptions;
	}

	public String getUsernameLast() {
		return usernameLast;
	}

	public void setUsernameLast(String usernameLast) {
		this.usernameLast = usernameLast;
	}
	
}