package uoc.com.mimenudehoy.model.validator;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uoc.com.mimenudehoy.model.User;
import uoc.com.mimenudehoy.service.UserService;

@Component
public class UserValidator implements Validator {

	@Autowired
	private UserService userService;

	public boolean supports(Class clazz) {
		return User.class.equals(clazz);
	}

	public void validate(Object target, Errors errors) {
		User user = (User) target;

		if(user.getUsername() == null) {
			errors.rejectValue("username", "error.required");
		}

		// verify username not exists when editing
		if(user.getUsernameLast()!=null && !user.getUsernameLast().isEmpty() && !user.getUsername().equals(user.getUsernameLast())){
			
			User tmp = userService.getUser(user.getUsername());
			if( Hibernate.isInitialized(user) && tmp!=null ){
				errors.rejectValue("username", "user.error.username_exists");
			}
			
		}

		if( user.getPassword().isEmpty() ){
			errors.rejectValue("password", "error.required");
		}

		//verify user has one role at least
		if( user.getRoles()==null || user.getRoles().size()==0){
			errors.rejectValue("roles", "error.required");
		}

	}

}
