package uoc.com.mimenudehoy.service.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import uoc.com.mimenudehoy.dao.MenuDailyDao;
import uoc.com.mimenudehoy.model.MenuDaily;
import uoc.com.mimenudehoy.model.Subscription;
import uoc.com.mimenudehoy.service.MenuDailyService;

@Service
public class MenuDailyServiceImpl implements MenuDailyService {

	@Autowired
	private MenuDailyDao dao;
	
	
	@Transactional
	public void save(MenuDaily user) {
		dao.save(user);
	}

	@Transactional
	public void delete(String id ) {
		dao.delete(id);
	}

	@Transactional(readOnly = true)
	public MenuDaily find(String id) {
		return dao.findById(id);
	}

	@Transactional(readOnly = true)
	public List<MenuDaily> getDailyMenusOfThisWeek(Subscription subs) {
		
		Date hoy = new Date();
		
		Calendar lunes = Calendar.getInstance();
		lunes.setTime(hoy);
		lunes.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
		
		lunes.set(Calendar.HOUR, 0);
		lunes.set(Calendar.MINUTE, 0);
		lunes.set(Calendar.SECOND, 0);
		lunes.set(Calendar.MILLISECOND, 0);
		
		Calendar domingo = Calendar.getInstance();
		domingo.setTime(hoy);
		domingo.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
		
		domingo.set(Calendar.HOUR, 0);
		domingo.set(Calendar.MINUTE, 0);
		domingo.set(Calendar.SECOND, 0);
		domingo.set(Calendar.MILLISECOND, 0);
		
		return dao.findByPeriod(lunes.getTime(), domingo.getTime(),subs);
	}

	@Transactional(readOnly = true)
	public List<MenuDaily> getDailyMenusOfNextWeek(Subscription subs) {
		
		Date hoy = new Date();
		
		Calendar lunes = Calendar.getInstance();
		lunes.setTime(hoy);
		lunes.add(Calendar.WEEK_OF_YEAR, 1);
		lunes.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);

		lunes.set(Calendar.HOUR, 0);
		lunes.set(Calendar.MINUTE, 0);
		lunes.set(Calendar.SECOND, 0);
		lunes.set(Calendar.MILLISECOND, 0);

		
		Calendar domingo = Calendar.getInstance();
		domingo.setTime(hoy);
		domingo.add(Calendar.WEEK_OF_YEAR, 1);
		domingo.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
		
		domingo.set(Calendar.HOUR, 0);
		domingo.set(Calendar.MINUTE, 0);
		domingo.set(Calendar.SECOND, 0);
		domingo.set(Calendar.MILLISECOND, 0);
		
		return dao.findByPeriod(lunes.getTime(), domingo.getTime(),subs);
	}
	
}
