package uoc.com.mimenudehoy.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import uoc.com.mimenudehoy.dao.RecipeDao;
import uoc.com.mimenudehoy.model.EnumCuisineType;
import uoc.com.mimenudehoy.model.Recipe;
import uoc.com.mimenudehoy.service.RecipeService;

@Service
public class RecipeServiceImpl implements RecipeService {

	@Autowired
	private RecipeDao dao;

	@Transactional
	public void save(Recipe user) {
		dao.save(user);
	}

	@Transactional(readOnly = true)
	public List<Recipe> listRecipes() {
		return dao.list();
	}

	@Transactional(readOnly = true)
	public List<Recipe> getLastRecipes() {
		return dao.getMostRecent(10);
	}	
	
	@Transactional(readOnly = true)
	public List<Recipe> searchRecipes(String text,EnumCuisineType tc) {
		return dao.findBy(text,tc);
	}

	@Transactional
	public void delete(String id ) {
		dao.delete(id);
	}

	@Transactional(readOnly = true)
	public Recipe getRecipe(String id) {
		return dao.findById(id);
	}

	@Transactional(readOnly = true)
	public List<Recipe> getHighLightsLateral() {
		return dao.getHighLights(false, false, true);
	}

	@Transactional(readOnly = true)
	public List<Recipe> getHighLightsFooter() {
		List<Recipe> result = dao.getHighLights(false, true, false);
		if(result.size()>3) result = result.subList(0, 2); 
		return result;
	}

	@Transactional(readOnly = true)
	public Recipe getHighLightMain() {
		List<Recipe> list =dao.getHighLights(true, false, false);
		if(list.size()==0) return null;
		return list.get(0);
	}

}
