package uoc.com.mimenudehoy.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.TreeMap;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import uoc.com.mimenudehoy.dao.SubscriptionDao;
import uoc.com.mimenudehoy.model.Menu;
import uoc.com.mimenudehoy.model.MenuDaily;
import uoc.com.mimenudehoy.model.Subscription;
import uoc.com.mimenudehoy.model.SubscriptionAssign;
import uoc.com.mimenudehoy.model.User;
import uoc.com.mimenudehoy.service.SubscriptionsService;
import uoc.com.mimenudehoy.sort.MenuDailyComparatorDate;

@Service
public class SubscriptionServiceImpl implements SubscriptionsService {

	@Autowired
	private SubscriptionDao dao;

	@Transactional
	public void save(Subscription obj) {
		dao.save(obj);
	}

	@Transactional(readOnly = true)
	public List<Subscription> list() {
		return dao.list();
	}

	@Transactional
	public void delete(String id ) {
		dao.delete(id);
	}

	@Transactional(readOnly = true)
	public Subscription getSubscription(String id) {
		return dao.findById(id);
	}

	@Transactional(readOnly = true)
	public List<Subscription> listActives() {
		return dao.findActives();
	}

	@Transactional(readOnly = true)
	public List<Subscription> listActivesByUser(User user) {
		return dao.findActiveByUser(user);
	}

	
	@Transactional(readOnly = true)
	public List<SubscriptionAssign> listMissingMenus() {
		List<SubscriptionAssign> lsa = new ArrayList<SubscriptionAssign>(); 
		
		List<Subscription> all =dao.findMissingMenus(); 
		for(Subscription subs:all){
			lsa.add(new SubscriptionAssign(subs));
		}
		
		return lsa;
	}

	@Transactional(readOnly = true)
	public List<MenuDaily> getAllDailyMenus(String id_subscription) {
		
		TreeMap<Date, MenuDaily> tm = new TreeMap<Date, MenuDaily>();
		
		Subscription subs = dao.findById(id_subscription);
		
		//fill current menus
		if(subs.getMenus().size()>0){
			for(MenuDaily md:subs.getMenus()){
				tm.put(md.getDate(),md);
			}
		}
		
		Calendar start = Calendar.getInstance();
		start.setTime(subs.getStart_date());
		start.set(Calendar.HOUR, 0);
		start.set(Calendar.MINUTE, 0);
		start.set(Calendar.SECOND, 0);
		start.set(Calendar.MILLISECOND, 0);

		Calendar end = Calendar.getInstance();
		end.setTime(subs.getEnd_date());
		end.set(Calendar.HOUR, 0);
		end.set(Calendar.MINUTE, 0);
		end.set(Calendar.SECOND, 0);
		end.set(Calendar.MILLISECOND, 0);

		while(!start.after(end)){
			
			Date current = start.getTime();
			
			if(!tm.containsKey(current)){

				MenuDaily md = new MenuDaily();
				md.setDate(start.getTime());
				tm.put(current, md);

			}
			
			start.add(Calendar.DAY_OF_YEAR, 1);
		}
		
		
		return new ArrayList<MenuDaily>(tm.values());
	}


}
